require "isbn-test"
local ISBN = require "isbn"

module("test.isbn13", lunit.testcase, package.seeall)

local GOOD_ISBN          = "9780596527242"
local GOOD_ISBN_STRING   = "978-0-596-52724-2"
local GOOD_ISBN10        = "0596527241"
local PREFIX             = "978"
local GROUP_NAME         = "English speaking area"
local GROUP_CODE         = "0"
local PUBLISHER          = "596"
local ARTICLE            = "52724"
local CHECKSUM           = "2"

local BAD_CHECKSUM_ISBN  = "9780596527244"
local X_BAD_CHECKSUM_ISBN = "978059652724X"
local MISSING_CHECKSUM_ISBN = "978059652724"

function setup ()
    local err
    isbn, err = ISBN(GOOD_ISBN)
    assert(not err)
end

function test_construct_object ()
    assert_table(isbn)
    assert_isa(isbn, ISBN)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(PREFIX, isbn:prefix(), "has right prefix")
    is(GROUP_CODE, isbn:group_code(), "has right group code")
    is(GROUP_NAME, isbn:group_name(), "has right group name")
    is(PUBLISHER, isbn:publisher_code(), "has right publisher")
    is(ARTICLE, isbn:article_code(), "has right article")
    is(CHECKSUM, isbn:checksum(), "has right checksum")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringify metamethod")
end

-- can I clone it?
function test_clone ()
    local clone = isbn:as_isbn13()
    assert_isa(clone, ISBN)
    is(GOOD_ISBN, clone:isbn(), "isbn() method")
    is(PREFIX, clone:prefix(), "has right prefix")
    is(GROUP_CODE, clone:group_code(), "has right group code")
    is(GROUP_NAME, clone:group_name(), "has right group name")
    is(PUBLISHER, clone:publisher_code(), "has right publisher")
    is(ARTICLE, clone:article_code(), "has right article")
    is(CHECKSUM, clone:checksum(), "has right checksum")
    is(GOOD_ISBN_STRING, tostring(clone), "stringifies correctly")
end

-- can I make it ISBN10?
function test_isbn10 ()
    local clone = isbn:as_isbn10()
    assert_isa(clone, ISBN)
    is(GOOD_ISBN10, clone:isbn(), "isbn() method")
    is("", clone:prefix(), "has right prefix")
    is(PUBLISHER, clone:publisher_code(), "has right publisher")
    is(GROUP_CODE, clone:group_code(), "has right country code")
    is(GROUP_NAME, clone:group_name(), "has right country name")
end

function test_bad_prefix ()
    local isbn, err = ISBN("1230439994149")
    assert_nil(isbn)
    is("invalid prefix", err)
end

function test_completely_wrong ()
    local isbn, err = ISBN("978abcdefghij")
    assert_nil(isbn)
    is("wrong number of digits", err)
    local isbn, err = ISBN("978156592")
    assert_nil(isbn)
    is("wrong number of digits", err)
end

function test_x_in_middle ()
    local isbn, err = ISBN("97805X6527242")
    assert_nil(isbn)
    is("digit 'X' not allowed in middle", err)
end

function test_bad_checksum ()
    local isbn, err = ISBN(BAD_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong checksum digit", err)
    isbn, err = ISBN(BAD_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, ISBN)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function test_bad_checksum_x ()
    -- The check digit 'X' is never correct on an ISBN-13, but it should
    -- be treated as a wrong checksum, and therefore can be corrected if
    -- you ask nicely.
    local isbn, err = ISBN(X_BAD_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong checksum digit", err)
    isbn, err = ISBN(X_BAD_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, ISBN)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function test_missing_checksums ()
    local isbn, err = ISBN(MISSING_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong number of digits", err)
    isbn, err = ISBN(MISSING_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, ISBN)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function test_bad_group_code ()
    local isbn, err = ISBN("978-9990222576")
    assert_nil(isbn)
    is("invalid group code", err)
end

function test_bad_publisher_code ()
    local isbn, err = ISBN("978-9165022222")
    assert_nil(isbn)
    is("invalid publisher code", err)
end

function test_strict_and_lenient ()
    -- These extra characters are normally ignored, but not in strict mode.
    local GOOD_WITH_BAD_CHARS = " 978-0596%527  (ignored) 242\t\n\r"
    local isbn, err = ISBN(GOOD_WITH_BAD_CHARS)
    is(GOOD_ISBN, isbn:isbn())
    isbn, err = ISBN(GOOD_WITH_BAD_CHARS, "strict")
    assert_nil(isbn)
    is("character not allowed", err)

    -- These extra characters are always allowed, even in strict mode.
    local GOOD_WITH_SUPERFLUOUS_CHARS = " 978-0596 527- - 242\t\n\r"
    local isbn, err = ISBN(GOOD_WITH_SUPERFLUOUS_CHARS)
    is(GOOD_ISBN, isbn:isbn())
    isbn, err = ISBN(GOOD_WITH_SUPERFLUOUS_CHARS, "strict")
    is(GOOD_ISBN, isbn:isbn())
end

function test_good_isbns ()
    local file = assert(io.open("test/data/isbn13s.txt", "rb"))

    for line in file:lines() do
        local isbn, err = ISBN(line)
        assert_table(isbn)
        assert_nil(err)
    end

    file:close()
end

function test_bad_isbns ()
    local file = assert(io.open("test/data/bad-isbn13s.txt", "rb"))

    for line in file:lines() do
        local isbn, err = ISBN(line)
        assert_nil(isbn, "value returned for bad ISBN-13: " .. line)
        assert_string(err, "no string for error message for ISBN-13: " .. line)
    end

    file:close()
end

-- vi:ts=4 sw=4 expandtab
